@@ -269,7 +269,7 @@ type URI = string
 
 	types["LSPAny"] = "type LSPAny = interface{}\n"
 	// A special case, the only previously existing Or type
-	types["DocumentDiagnosticReport"] = "type DocumentDiagnosticReport = Or_DocumentDiagnosticReport // (alias) line 13909\n"
+	types["DocumentDiagnosticReport"] = "type DocumentDiagnosticReport = Or_DocumentDiagnosticReport // (alias) \n"
 
 }
 
@@ -300,7 +300,7 @@ func genAliases(model Model) {
 			continue // renamed the type, e.g., "DocumentDiagnosticReport", an or-type to "string"
 		}
 		tp := goplsName(ta.Type)
-		fmt.Fprintf(out, "type %s = %s // (alias) line %d\n", nm, tp, ta.Line)
+		fmt.Fprintf(out, "type %s = %s // (alias)\n", nm, tp)
 		types[nm] = out.String()
 	}
 }
@@ -395,7 +395,6 @@ func genMarshal() {
 		}
 		sort.Strings(names)
 		var buf bytes.Buffer
-		fmt.Fprintf(&buf, "// from line %d\n", nt.line)
 		fmt.Fprintf(&buf, "func (t %s) MarshalJSON() ([]byte, error) {\n", nm)
 		buf.WriteString("\tswitch x := t.Value.(type){\n")
 		for _, nmx := range names {
