@@ -12,10 +12,9 @@ import (
 	"unicode"
 
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
-	"golang.org/x/tools/gopls/internal/lsp/snippet"
 	"golang.org/x/tools/gopls/internal/lsp/source"
+	"golang.org/x/tools/gopls/internal/lsp/source/completion/snippet"
 	"golang.org/x/tools/internal/event"
-	"golang.org/x/tools/internal/typeparams"
 )
 
 // literal generates composite literal, function literal, and make()
@@ -194,7 +193,7 @@ func (c *completer) functionLiteral(ctx context.Context, sig *types.Signature, m
 			name = p.Name()
 		)
 
-		if tp, _ := p.Type().(*typeparams.TypeParam); tp != nil && !c.typeParamInScope(tp) {
+		if tp, _ := p.Type().(*types.TypeParam); tp != nil && !c.typeParamInScope(tp) {
 			hasTypeParams = true
 		}
 
@@ -285,7 +284,7 @@ func (c *completer) functionLiteral(ctx context.Context, sig *types.Signature, m
 				typeStr = strings.Replace(typeStr, "[]", "...", 1)
 			}
 
-			if tp, _ := p.Type().(*typeparams.TypeParam); tp != nil && !c.typeParamInScope(tp) {
+			if tp, _ := p.Type().(*types.TypeParam); tp != nil && !c.typeParamInScope(tp) {
 				snip.WritePlaceholder(func(snip *snippet.Builder) {
 					snip.WriteText(typeStr)
 				})
@@ -306,7 +305,7 @@ func (c *completer) functionLiteral(ctx context.Context, sig *types.Signature, m
 
 	var resultHasTypeParams bool
 	for i := 0; i < results.Len(); i++ {
-		if tp, _ := results.At(i).Type().(*typeparams.TypeParam); tp != nil && !c.typeParamInScope(tp) {
+		if tp, _ := results.At(i).Type().(*types.TypeParam); tp != nil && !c.typeParamInScope(tp) {
 			resultHasTypeParams = true
 		}
 	}
@@ -339,7 +338,7 @@ func (c *completer) functionLiteral(ctx context.Context, sig *types.Signature, m
 			}
 			return
 		}
-		if tp, _ := r.Type().(*typeparams.TypeParam); tp != nil && !c.typeParamInScope(tp) {
+		if tp, _ := r.Type().(*types.TypeParam); tp != nil && !c.typeParamInScope(tp) {
 			snip.WritePlaceholder(func(snip *snippet.Builder) {
 				snip.WriteText(text)
 			})
@@ -513,7 +512,7 @@ func (c *completer) typeNameSnippet(literalType types.Type, qf types.Qualifier)
 		named, _ = literalType.(*types.Named)
 	)
 
-	if named != nil && named.Obj() != nil && typeparams.ForNamed(named).Len() > 0 && !c.fullyInstantiated(named) {
+	if named != nil && named.Obj() != nil && named.TypeParams().Len() > 0 && !c.fullyInstantiated(named) {
 		// We are not "fully instantiated" meaning we have type params that must be specified.
 		if pkg := qf(named.Obj().Pkg()); pkg != "" {
 			typeName = pkg + "."
@@ -524,12 +523,12 @@ func (c *completer) typeNameSnippet(literalType types.Type, qf types.Qualifier)
 		snip.WriteText(typeName + "[")
 
 		if c.opts.placeholders {
-			for i := 0; i < typeparams.ForNamed(named).Len(); i++ {
+			for i := 0; i < named.TypeParams().Len(); i++ {
 				if i > 0 {
 					snip.WriteText(", ")
 				}
 				snip.WritePlaceholder(func(snip *snippet.Builder) {
-					snip.WriteText(types.TypeString(typeparams.ForNamed(named).At(i), qf))
+					snip.WriteText(types.TypeString(named.TypeParams().At(i), qf))
 				})
 			}
 		} else {
@@ -549,8 +548,8 @@ func (c *completer) typeNameSnippet(literalType types.Type, qf types.Qualifier)
 // fullyInstantiated reports whether all of t's type params have
 // specified type args.
 func (c *completer) fullyInstantiated(t *types.Named) bool {
-	tps := typeparams.ForNamed(t)
-	tas := typeparams.NamedTypeArgs(t)
+	tps := t.TypeParams()
+	tas := t.TypeArgs()
 
 	if tps.Len() != tas.Len() {
 		return false
@@ -558,7 +557,7 @@ func (c *completer) fullyInstantiated(t *types.Named) bool {
 
 	for i := 0; i < tas.Len(); i++ {
 		switch ta := tas.At(i).(type) {
-		case *typeparams.TypeParam:
+		case *types.TypeParam:
 			// A *TypeParam only counts as specified if it is currently in
 			// scope (i.e. we are in a generic definition).
 			if !c.typeParamInScope(ta) {
@@ -576,7 +575,7 @@ func (c *completer) fullyInstantiated(t *types.Named) bool {
 // typeParamInScope returns whether tp's object is in scope at c.pos.
 // This tells you whether you are in a generic definition and can
 // assume tp has been specified.
-func (c *completer) typeParamInScope(tp *typeparams.TypeParam) bool {
+func (c *completer) typeParamInScope(tp *types.TypeParam) bool {
 	obj := tp.Obj()
 	if obj == nil {
 		return false
