@@ -5,6 +5,7 @@
 package source
 
 import (
+	"bytes"
 	"context"
 	"encoding/json"
 	"fmt"
@@ -24,14 +25,15 @@ import (
 	"golang.org/x/text/unicode/runenames"
 	"golang.org/x/tools/go/ast/astutil"
 	"golang.org/x/tools/go/types/typeutil"
-	"golang.org/x/tools/gopls/internal/bug"
 	"golang.org/x/tools/gopls/internal/file"
+	"golang.org/x/tools/gopls/internal/lsp/cache"
+	"golang.org/x/tools/gopls/internal/lsp/cache/metadata"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
-	"golang.org/x/tools/gopls/internal/lsp/safetoken"
 	"golang.org/x/tools/gopls/internal/settings"
+	"golang.org/x/tools/gopls/internal/util/bug"
+	"golang.org/x/tools/gopls/internal/util/safetoken"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/tokeninternal"
-	"golang.org/x/tools/internal/typeparams"
 )
 
 // HoverJSON contains information used by hover. It is also the JSON returned
@@ -63,7 +65,7 @@ type HoverJSON struct {
 }
 
 // Hover implements the "textDocument/hover" RPC for Go files.
-func Hover(ctx context.Context, snapshot Snapshot, fh file.Handle, position protocol.Position) (*protocol.Hover, error) {
+func Hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, position protocol.Position) (*protocol.Hover, error) {
 	ctx, done := event.Start(ctx, "source.Hover")
 	defer done()
 
@@ -90,7 +92,7 @@ func Hover(ctx context.Context, snapshot Snapshot, fh file.Handle, position prot
 // hover computes hover information at the given position. If we do not support
 // hovering at the position, it returns _, nil, nil: an error is only returned
 // if the position is valid but we fail to compute hover information.
-func hover(ctx context.Context, snapshot Snapshot, fh file.Handle, pp protocol.Position) (protocol.Range, *HoverJSON, error) {
+func hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, pp protocol.Position) (protocol.Range, *HoverJSON, error) {
 	pkg, pgf, err := NarrowestPackageForFile(ctx, snapshot, fh.URI())
 	if err != nil {
 		return protocol.Range{}, nil, err
@@ -183,7 +185,7 @@ func hover(ctx context.Context, snapshot Snapshot, fh file.Handle, pp protocol.P
 	}
 
 	// Handle builtins, which don't have a package or position.
-	if obj.Pkg() == nil {
+	if !obj.Pos().IsValid() {
 		h, err := hoverBuiltin(ctx, snapshot, obj)
 		return rng, h, err
 	}
@@ -218,8 +220,10 @@ func hover(ctx context.Context, snapshot Snapshot, fh file.Handle, pp protocol.P
 	//
 	// TODO(rfindley): this should use FormatVarType to get proper qualification
 	// of identifiers, and we should revisit the formatting of method set.
+	//
+	// TODO(adonovan): this logic belongs in objectString.
 	_, isTypeName := obj.(*types.TypeName)
-	_, isTypeParam := obj.Type().(*typeparams.TypeParam)
+	_, isTypeParam := obj.Type().(*types.TypeParam)
 	if isTypeName && !isTypeParam {
 		spec, ok := spec.(*ast.TypeSpec)
 		if !ok {
@@ -260,15 +264,15 @@ func hover(ctx context.Context, snapshot Snapshot, fh file.Handle, pp protocol.P
 	//
 	// If linkPath is empty, the symbol is not linkable.
 	var (
-		linkName string    // => link title, always non-empty
-		linkPath string    // => link path
-		anchor   string    // link anchor
-		linkMeta *Metadata // metadata for the linked package
+		linkName string            // => link title, always non-empty
+		linkPath string            // => link path
+		anchor   string            // link anchor
+		linkMeta *metadata.Package // metadata for the linked package
 	)
 	{
 		linkMeta = findFileInDeps(snapshot, pkg.Metadata(), declPGF.URI)
 		if linkMeta == nil {
-			return protocol.Range{}, nil, bug.Errorf("no metadata for %s", declPGF.URI)
+			return protocol.Range{}, nil, bug.Errorf("no package data for %s", declPGF.URI)
 		}
 
 		// For package names, we simply link to their imported package.
@@ -278,7 +282,10 @@ func hover(ctx context.Context, snapshot Snapshot, fh file.Handle, pp protocol.P
 			impID := linkMeta.DepsByPkgPath[PackagePath(pkgName.Imported().Path())]
 			linkMeta = snapshot.Metadata(impID)
 			if linkMeta == nil {
-				return protocol.Range{}, nil, bug.Errorf("no metadata for %s", declPGF.URI)
+				// Broken imports have fake package paths, so it is not a bug if we
+				// don't have metadata. As of writing, there is no way to distinguish
+				// broken imports from a true bug where expected metadata is missing.
+				return protocol.Range{}, nil, fmt.Errorf("no package data for %s", declPGF.URI)
 			}
 		} else {
 			// For all others, check whether the object is in the package scope, or
@@ -366,13 +373,10 @@ func hover(ctx context.Context, snapshot Snapshot, fh file.Handle, pp protocol.P
 
 // hoverBuiltin computes hover information when hovering over a builtin
 // identifier.
-func hoverBuiltin(ctx context.Context, snapshot Snapshot, obj types.Object) (*HoverJSON, error) {
-	// TODO(rfindley): link to the correct version of Go documentation.
-	builtin, err := snapshot.BuiltinFile(ctx)
-	if err != nil {
-		return nil, err
-	}
-
+func hoverBuiltin(ctx context.Context, snapshot *cache.Snapshot, obj types.Object) (*HoverJSON, error) {
+	// Special handling for error.Error, which is the only builtin method.
+	//
+	// TODO(rfindley): can this be unified with the handling below?
 	if obj.Name() == "Error" {
 		signature := obj.String()
 		return &HoverJSON{
@@ -385,18 +389,13 @@ func hoverBuiltin(ctx context.Context, snapshot Snapshot, obj types.Object) (*Ho
 		}, nil
 	}
 
-	builtinObj := builtin.File.Scope.Lookup(obj.Name())
-	if builtinObj == nil {
-		// All builtins should have a declaration in the builtin file.
-		return nil, bug.Errorf("no builtin object for %s", obj.Name())
-	}
-	node, _ := builtinObj.Decl.(ast.Node)
-	if node == nil {
-		return nil, bug.Errorf("no declaration for %s", obj.Name())
+	pgf, node, err := builtinDecl(ctx, snapshot, obj)
+	if err != nil {
+		return nil, err
 	}
 
 	var comment *ast.CommentGroup
-	path, _ := astutil.PathEnclosingInterval(builtin.File, node.Pos(), node.End())
+	path, _ := astutil.PathEnclosingInterval(pgf.File, node.Pos(), node.End())
 	for _, n := range path {
 		switch n := n.(type) {
 		case *ast.GenDecl:
@@ -414,7 +413,7 @@ func hoverBuiltin(ctx context.Context, snapshot Snapshot, obj types.Object) (*Ho
 		}
 	}
 
-	signature := FormatNodeFile(builtin.Tok, node)
+	signature := FormatNodeFile(pgf.Tok, node)
 	// Replace fake types with their common equivalent.
 	// TODO(rfindley): we should instead use obj.Type(), which would have the
 	// *actual* types of the builtin call.
@@ -436,13 +435,13 @@ func hoverBuiltin(ctx context.Context, snapshot Snapshot, obj types.Object) (*Ho
 // imp in the file pgf of pkg.
 //
 // If we do not have metadata for the hovered import, it returns _
-func hoverImport(ctx context.Context, snapshot Snapshot, pkg Package, pgf *ParsedGoFile, imp *ast.ImportSpec) (protocol.Range, *HoverJSON, error) {
+func hoverImport(ctx context.Context, snapshot *cache.Snapshot, pkg *cache.Package, pgf *ParsedGoFile, imp *ast.ImportSpec) (protocol.Range, *HoverJSON, error) {
 	rng, err := pgf.NodeRange(imp.Path)
 	if err != nil {
 		return protocol.Range{}, nil, err
 	}
 
-	importPath := UnquoteImportPath(imp)
+	importPath := metadata.UnquoteImportPath(imp)
 	if importPath == "" {
 		return protocol.Range{}, nil, fmt.Errorf("invalid import path")
 	}
@@ -487,7 +486,7 @@ func hoverImport(ctx context.Context, snapshot Snapshot, pkg Package, pgf *Parse
 
 // hoverPackageName computes hover information for the package name of the file
 // pgf in pkg.
-func hoverPackageName(pkg Package, pgf *ParsedGoFile) (protocol.Range, *HoverJSON, error) {
+func hoverPackageName(pkg *cache.Package, pgf *ParsedGoFile) (protocol.Range, *HoverJSON, error) {
 	var comment *ast.CommentGroup
 	for _, pgf := range pkg.CompiledGoFiles() {
 		if pgf.File.Doc != nil {
@@ -591,8 +590,8 @@ func hoverLit(pgf *ParsedGoFile, lit *ast.BasicLit, pos token.Pos) (protocol.Ran
 					return protocol.Range{}, nil, nil
 				}
 				// Only the rune escape sequence part of the string has to be highlighted, recompute the range.
-				runeLen := len(lit.Value) - (int(i) + len(tail))
-				start = token.Pos(int(lit.Pos()) + int(i))
+				runeLen := len(lit.Value) - (i + len(tail))
+				start = token.Pos(int(lit.Pos()) + i)
 				end = token.Pos(int(start) + runeLen)
 				break
 			}
@@ -682,7 +681,7 @@ func hoverEmbed(fh file.Handle, rng protocol.Range, pattern string) (protocol.Ra
 func inferredSignatureString(obj types.Object, qf types.Qualifier, inferred *types.Signature) string {
 	// If the signature type was inferred, prefer the inferred signature with a
 	// comment showing the generic signature.
-	if sig, _ := obj.Type().(*types.Signature); sig != nil && typeparams.ForSignature(sig).Len() > 0 && inferred != nil {
+	if sig, _ := obj.Type().(*types.Signature); sig != nil && sig.TypeParams().Len() > 0 && inferred != nil {
 		obj2 := types.NewFunc(obj.Pos(), obj.Pkg(), obj.Name(), inferred)
 		str := types.ObjectString(obj2, qf)
 		// Try to avoid overly long lines.
@@ -701,11 +700,50 @@ func inferredSignatureString(obj types.Object, qf types.Qualifier, inferred *typ
 // It handles adding more information to the object string.
 // If spec is non-nil, it may be used to format additional declaration
 // syntax, and file must be the token.File describing its positions.
+//
+// Precondition: obj is not a built-in function or method.
 func objectString(obj types.Object, qf types.Qualifier, declPos token.Pos, file *token.File, spec ast.Spec) string {
 	str := types.ObjectString(obj, qf)
 
 	switch obj := obj.(type) {
+	case *types.Func:
+		// We fork ObjectString to improve its rendering of methods:
+		// specifically, we show the receiver name,
+		// and replace the period in (T).f by a space (#62190).
+
+		sig := obj.Type().(*types.Signature)
+
+		var buf bytes.Buffer
+		buf.WriteString("func ")
+		if recv := sig.Recv(); recv != nil {
+			buf.WriteByte('(')
+			if _, ok := recv.Type().(*types.Interface); ok {
+				// gcimporter creates abstract methods of
+				// named interfaces using the interface type
+				// (not the named type) as the receiver.
+				// Don't print it in full.
+				buf.WriteString("interface")
+			} else {
+				// Show receiver name (go/types does not).
+				name := recv.Name()
+				if name != "" && name != "_" {
+					buf.WriteString(name)
+					buf.WriteString(" ")
+				}
+				types.WriteType(&buf, recv.Type(), qf)
+			}
+			buf.WriteByte(')')
+			buf.WriteByte(' ') // space (go/types uses a period)
+		} else if s := qf(obj.Pkg()); s != "" {
+			buf.WriteString(s)
+			buf.WriteString(".")
+		}
+		buf.WriteString(obj.Name())
+		types.WriteSignature(&buf, sig, qf)
+		str = buf.String()
+
 	case *types.Const:
+		// Show value of a constant.
 		var (
 			declaration = obj.Val().String() // default formatted declaration
 			comment     = ""                 // if non-empty, a clarifying comment
@@ -761,9 +799,9 @@ func objectString(obj types.Object, qf types.Qualifier, declPos token.Pos, file
 // fset provides file/line information).
 //
 // TODO(rfindley): there appears to be zero(!) tests for this functionality.
-func HoverDocForObject(ctx context.Context, snapshot Snapshot, fset *token.FileSet, obj types.Object) (*ast.CommentGroup, error) {
+func HoverDocForObject(ctx context.Context, snapshot *cache.Snapshot, fset *token.FileSet, obj types.Object) (*ast.CommentGroup, error) {
 	if _, isTypeName := obj.(*types.TypeName); isTypeName {
-		if _, isTypeParam := obj.Type().(*typeparams.TypeParam); isTypeParam {
+		if _, isTypeParam := obj.Type().(*types.TypeParam); isTypeParam {
 			return nil, nil
 		}
 	}
@@ -818,7 +856,7 @@ func chooseDocComment(decl ast.Decl, spec ast.Spec, field *ast.Field) *ast.Comme
 //
 // It returns the resulting ParsedGoFile as well as new pos contained in the
 // parsed file.
-func parseFull(ctx context.Context, snapshot Snapshot, fset *token.FileSet, pos token.Pos) (*ParsedGoFile, token.Pos, error) {
+func parseFull(ctx context.Context, snapshot *cache.Snapshot, fset *token.FileSet, pos token.Pos) (*ParsedGoFile, token.Pos, error) {
 	f := fset.File(pos)
 	if f == nil {
 		return nil, 0, bug.Errorf("internal error: no file for position %d", pos)
@@ -899,7 +937,7 @@ func formatLink(h *HoverJSON, options *settings.Options) string {
 	if !options.LinksInHover || options.LinkTarget == "" || h.LinkPath == "" {
 		return ""
 	}
-	plainLink := BuildLink(options.LinkTarget, h.LinkPath, h.LinkAnchor)
+	plainLink := cache.BuildLink(options.LinkTarget, h.LinkPath, h.LinkAnchor)
 	switch options.PreferredContentFormat {
 	case protocol.Markdown:
 		return fmt.Sprintf("[`%s` on %s](%s)", h.SymbolName, options.LinkTarget, plainLink)
@@ -910,15 +948,6 @@ func formatLink(h *HoverJSON, options *settings.Options) string {
 	}
 }
 
-// BuildLink constructs a URL with the given target, path, and anchor.
-func BuildLink(target, path, anchor string) string {
-	link := fmt.Sprintf("https://%s/%s", target, path)
-	if anchor == "" {
-		return link
-	}
-	return link + "#" + anchor
-}
-
 func formatDoc(h *HoverJSON, options *settings.Options) string {
 	var doc string
 	switch options.HoverKind {
